/*
	HtmlImageView.js

	Created August 2011
	developer@martinpearman.co.uk
	
	A draggable and zoomable image viewer for B4A devices.
	
	Version 1.0
*/

/*	HtmlImageView()	Constructor
	
	imagePanel				Required	The DOM element that will be the parent of the HtmlImageView.
	
	config					Optional	An onject literal with properties:
	
	eventName				Optional	These two properties if defined must BOTH be defined.
	namespace							eventName is like a B4A View EventName and used to prefix calls to B4A Subs.
										namespace is the javascript namespace defined when using the JSInterface library addJSInterface() method.
										**	The JSInterface library is required to use these two properties	**
								
	disableFitImageToView	Optional	If an imageURL property is defined and this property is defined as true then the image will not be zoomed to completely fill the View.
	
	imageURL				Optional	The URL of an image to load.
	
	x						Optional	Define the pixel which to center the image on and also the View's initial zoom level.
	y									These three properties take priority over the disableFitImageToView property.
	zoom								So if disableFitImageToView is defined (as true), and also x, y and zoom are defined then the  x, y and zoom properties will be used.
*/
function HtmlImageView(imagePanel, config){
	function getQueryObject(){
		//	getQueryObject() returns an object literal of name/values from the page's GET parameters
		var $query=location.search, $args={};
		$query=$query.length>0?$query.substring(1):'';
		var $items=$query.split('&'), $item, $name, $value, $i;
		for($i=0; $i<$items.length; $i++){
			$item=$items[$i].split('=');
			$name=decodeURIComponent($item[0]);
			$value=decodeURIComponent($item[1]);
			$args[$name]=$value;
		}
		return $args;
	}
	function waitForDOM(){
		if(document.documentElement.clientWidth>0 && document.documentElement.clientHeight>0){
			$this.VIEWPORT_SIZE={width:document.documentElement.clientWidth, height:document.documentElement.clientHeight};
			$this.VIEWPORT_CENTER={x:parseInt($this.VIEWPORT_SIZE.width/2), y:parseInt($this.VIEWPORT_SIZE.height/2)};
			if($this.callback){
				$this.callback('DOMReady');
			}
			if(config.imageURL){
				$this.setViewState(config);
			}
		} else {
			setTimeout(waitForDOM, 250);
		}
	}
	imagePanel.style.position='absolute';
	config=config || {};
	var query=getQueryObject();
	for(var name in query){
		config[name]=query[name];
	}
	if(config.namespace && config.eventName){
		var namespace=config.namespace, eventName=config.eventName;
		this.callback=function($event, values){
			values=values || [];
			var javascript=['"'+eventName+'_'+$event+'"'];
			for(var i=0, length=values.length; i<length; i++){
				javascript.push('"'+values[i]+'"');
			}
			javascript=namespace+'.CallSub('+javascript.join(', ')+')';
			eval(javascript);
		};
	}
	this.image=null;
	this.IMAGE_PANEL=imagePanel;
	this.touchPoints=[];
	//	defined how much the image will shrink or grow on a zoom level change
	//	not yet documented
	this.zoomScale=config.zoomScale || 1.25;
	
	var $this=this;
	waitForDOM();
}

HtmlImageView.prototype.draw=function(){
	if(this.image){
		var zoomFactor=Math.pow(this.zoomScale, this.zoomLevel);
		if(this.touchPoints.length===2){
			var startPoint=this.touchPoints[0];
			var endPoint=this.touchPoints[1];
			this.imageFocalPoint.x+=parseInt((startPoint.x-endPoint.x)/zoomFactor);
			this.imageFocalPoint.y+=parseInt((startPoint.y-endPoint.y)/zoomFactor);
			this.touchPoints.splice(0, 1);
		}
		var width=parseInt(this.imageSize.width*zoomFactor);
		var height=parseInt(this.imageSize.height*zoomFactor);
		this.IMAGE_PANEL.style.height=height+'px';
		this.IMAGE_PANEL.style.width=width+'px';
		var left=parseInt(this.VIEWPORT_CENTER.x-(this.imageFocalPoint.x*zoomFactor));
		var top=parseInt(this.VIEWPORT_CENTER.y-(this.imageFocalPoint.y*zoomFactor));
		this.IMAGE_PANEL.style.left=left+'px';
		this.IMAGE_PANEL.style.top=top+'px';
		if(this.callback){
			this.callback('StateChanged', [this.imageFocalPoint.x, this.imageFocalPoint.y, this.zoomLevel]);
		}
	}
};

/*
	setViewState() method
	
	See the Contructor documentation for details about the config object literal.
	eventName and namespace cannot be defined in a call to setViewState(), they must be defined in the Contructor.
*/
HtmlImageView.prototype.setViewState=function(config){
	var $this=this;
	if(this.image){
		this.image.parentNode.removeChild(this.image);
	} else {
		this.IMAGE_PANEL.addEventListener('touchstart', function($event){
			$event.preventDefault();
			if($event.targetTouches.length===1){
				var touch=$event.targetTouches[0];
				var now=new Date().getTime();
				if($this.touchPoints.length===1 && (now-$this.touchPoints[0].timestamp)<250){
					//	a double touch to zoom in is defined as two touches within 250ms
					$this.zoomLevel++;
					$this.touchPoints.push($this.VIEWPORT_CENTER);
					$this.draw();
				} else {
					$this.touchPoints=[{x:touch.pageX, y:touch.pageY, timestamp:now}];
				}
			}
		}, false);
		this.IMAGE_PANEL.addEventListener('touchmove', function ($event) {
			$event.preventDefault();
			if($event.targetTouches.length===1){
				var touch=$event.targetTouches[0];
				$this.touchPoints.push({x:touch.pageX, y:touch.pageY});
				$this.draw();
			}
		}, false);
		this.IMAGE_PANEL.addEventListener('touchend', function($event){
			$event.preventDefault();
		}, false);
	}
	var img=document.createElement('img');
	img.onload=function(){
		//	console.log('Successfully loaded image. imageURL: '+config.imageURL);
		this.style.border='none';
		this.style.height='100%';
		this.style.width='100%';
		$this.IMAGE_PANEL.style.height=this.height+'px';
		$this.IMAGE_PANEL.style.width=this.width+'px';
		$this.IMAGE_PANEL.appendChild(this);
		$this.image=this;
		$this.imageSize={width:this.width, height:this.height};
		
		if(typeof config.x!=='undefined' && typeof config.y!=='undefined'){
			$this.imageFocalPoint={x:config.x, y:config.y};
		} else {
			$this.imageFocalPoint={x:parseInt(this.width/2), y:parseInt(this.height/2)};
		}
		
		var zoomLevel=0, fitImageZoomLevel=false, zoomFactor;
		if(this.width>$this.VIEWPORT_SIZE.width || this.height>$this.VIEWPORT_SIZE.height){
			//	need to zoom out to fit the image
			while(fitImageZoomLevel===false){
				zoomLevel--;
				zoomFactor=Math.pow($this.zoomScale, zoomLevel);
				if(this.width*zoomFactor<=$this.VIEWPORT_SIZE.width && this.height*zoomFactor<=$this.VIEWPORT_SIZE.height){
					fitImageZoomLevel=zoomLevel;
				}
			}
		} else {
			//	need to zoom in to fit the image
			while(fitImageZoomLevel===false){
				zoomLevel++;
				zoomFactor=Math.pow($this.zoomScale, zoomLevel);
				if(this.width*zoomFactor>$this.VIEWPORT_SIZE.width || this.height*zoomFactor>$this.VIEWPORT_SIZE.height){
					fitImageZoomLevel=zoomLevel-1;
				}
			}
		}
		$this.fitImageZoomLevel=fitImageZoomLevel;
		
		if(typeof config.zoom!=='undefined'){
			$this.zoomLevel=config.zoom;
		} else {
			if(typeof config.disableFitImageToView!=='undefined' && config.disableFitImageToView===true){
				$this.zoomLevel=0;
			} else {
				$this.zoomLevel=fitImageZoomLevel;
			}
		}
		
		$this.draw();
	};
	img.onerror=function($event){
		if($this.callback){
			$this.callback('Error', ['Failed to load image. imageURL: '+config.imageURL]);
		}
	};
	//	console.log('Loading image. imageURL: '+config.imageURL);
	img.src=config.imageURL;
};

/*
	setZoom() method
	
	zoom	Required	Pass a numeric value to setZoom() to set that zoom level.
						Alternatively pass a string 'in', 'out' or 'fit' to zoom in a level, zoom out a level or to fit the image to the display size.
*/
HtmlImageView.prototype.setZoom=function(zoom){
	switch(zoom){
		case 'in':
			this.zoomLevel++;
			break;
		case 'out':
			this.zoomLevel--;
			break;
		case 'fit':
			this.zoomLevel=this.fitImageZoomLevel;
			this.imageFocalPoint={x:parseInt(this.imageSize.width/2), y:parseInt(this.imageSize.height/2)};
			break;
		default:
			if(!isNaN(zoom)){
				this.zoomLevel=zoom;
			}
	}
	this.draw();
};
